# HRI_Engine(main).py
#
# Copyright 2017 Eiichi Inohira
# Copyright 2019 Ryota Higashi 
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python 3
# For HRI Engine

"""Main_HRI_Engine_example
"""

from __future__ import print_function # 3⇒2系互換

import os
import sys
import xml.etree.ElementTree as ET
import threading
from concurrent.futures import ThreadPoolExecutor
import queue
import time
from datetime import datetime
from collections import deque
import logging
import platform
import importlib
import json
import itertools
import socketserver
import xmlrpc.server

from pyrois import RoIS_HRI, RoIS_Common, RoIS_Service
from . import message_profile, Service_Application_Base_sample, HRI_Engine_client_sample

class ThreadingXMLRPCServer(socketserver.ThreadingMixIn, xmlrpc.server.SimpleXMLRPCServer):
    """ThreadingXMLRPCServer
    """
    pass


class SystemIF(RoIS_HRI.SystemIF):
    """SystemIF
    """
    def __init__(self, Engine):
        self._engine = Engine

    def connect(self):
        self._engine.state = True
        # サブHRIエンジンはコンストラクタで行うconnectを実行する
        status = RoIS_HRI.ReturnCode_t.OK.value
        self.logger.info("connect")
        return status

    def disconnect(self):
        self._engine.state = False
        # サブHRIエンジンのdisconnectは実行しない
        status = RoIS_HRI.ReturnCode_t.OK.value
        self.logger.info("disconnect")
        return status

    def get_profile(self, condition):
        if self._engine.state is True:
            # プロファイルを返す(サブHRIエンジンと同じ) サブHRIエンジンのget_profileは呼ばない
            profile = self.engine_profile
            status = RoIS_HRI.ReturnCode_t.OK.value
            self.logger.info("Get_Profile:Success")
        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            profile = " "
            self.logger.error("Get_Profile:Engine_status_is_false")
        return (status, profile)

    def get_error_detail(self, error_id, condition):
        if self._engine.state is True:
            # error_idからHRIエンジンを判別し，そのエンジンのget_error_detailを実行する
            e = error_id.split('-')
            engine_name = e[0]
            # サブエンジンの場合
            if engine_name in self.sub_engine.keys():
                try:
                    (s, results) = self.sub_engine[engine_name].get_error_detail(error_id, condition)
                    status = s.value
                    if s == RoIS_HRI.ReturnCode_t.OK:
                        self.logger.info("Get_Error_Detail({}):Status_of_Get_Error_Detail_in_sub_engine_is_{}".format(error_id,s))
                    else:
                        self.logger.error("Get_Error_Detail({}):Status_of_Get_Error_Detail_in_sub_engine_is_{}".format(error_id,s))
                except:
                    status = RoIS_HRI.ReturnCode_t.ERROR.value
                    results = ["Cannot_connect_to_the_engine[{}]".format(e[0])]
                    self.logger.error("Get_Error_Detail({}):Cannot_connect_to_the_Get_Error_Detail_in_[{}]_engine".format(error_id,e[0]))
            # メインエンジン(自分自身)の場合
            elif engine_name == self.my_engine_name:
                if error_id in self.error_detail.keys():
                    status = RoIS_HRI.ReturnCode_t.OK.value
                    results = [self.error_detail[error_id]]
                    self.logger.info("Get_Error_Detail({}):Success <results:{}>".format(error_id,results))
                else:
                    status = RoIS_HRI.ReturnCode_t.ERROR.value
                    results = ["Error_id({})_is_wrong".format(error_id)]
                    self.logger.error("Get_Error_Detail({}):Error_id_is_wrong".format(error_id))
            else:
                status = RoIS_HRI.ReturnCode_t.ERROR.value
                results = ["Engine[{}]_is_not_registered".format(e[0])]
                self.logger.error("Get_Error_Detail({}):Engine[{}]_is_not_registered".format(error_id,e[0]))
        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            results = ["Engine_state_is_false"]
            self.logger.error("Get_Error_Detail({}):Engine_status_is_false".format(error_id))
        return (status, results)


class CommandIF(RoIS_HRI.CommandIF):
    """CommandIF
    """
    def __init__(self, Engine):
        self._engine = Engine

    def search(self, condition):
        command_id = self.create_new_id("command")
        if self._engine.state is True:
            component_ref_list = self.component_list
            status = RoIS_HRI.ReturnCode_t.OK.value
            self.command_result[command_id] = "Search:Success <component_ref_list:{}>".format(component_ref_list)
        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            component_ref_list = [" "]
            self.command_result[command_id] = "Search:Engine_status_is_false"
        
        if status == RoIS_HRI.ReturnCode_t.OK.value:
            self.logger.info("{}:{}".format(command_id,self.command_result[command_id]))
        else:
            self.logger.error("{}:{}".format(command_id,self.command_result[command_id]))
        return (status, component_ref_list)

    def bind(self, component_ref):
        command_id = self.create_new_id("command")
        if self._engine.state is True:
            if component_ref in self.component_list:
                # component_refからサブHRIエンジンを判別し，そのエンジンのbindを実行する
                m = component_ref.split(':')
                engine_name = m[5]
                # サブエンジンの場合
                if engine_name in self.sub_engine.keys():
                    try:
                        s = self.sub_engine[engine_name].bind(component_ref)
                        status = s.value
                        self.command_result[command_id] = "Bind({}):Status_of_bind_in_sub_engine_is_{}".format(component_ref,s)
                    except:
                        status = RoIS_HRI.ReturnCode_t.ERROR.value
                        self.command_result[command_id] = "Bind({}):Cannot_connect_to_the_bind_method_in_[{}]_engine".format(component_ref,m[5])
                # メインエンジン(自分自身)の場合
                elif engine_name == self.my_engine_name:
                    if component_ref in self.binding:
                        status = RoIS_HRI.ReturnCode_t.OK.value
                        self.command_result[command_id] = "Bind({}):Already_Bound".format(component_ref)
                    else:
                        self.binding.append(component_ref)
                        status = RoIS_HRI.ReturnCode_t.OK.value
                        self.command_result[command_id] = "Bind({}):Success".format(component_ref)
                # エンジンがない場合
                else:
                    status = RoIS_HRI.ReturnCode_t.ERROR.value
                    self.command_result[command_id] = "Bind({}):Engine[{}]_is_not_registered".format(component_ref,m[5])
            else:
                status = RoIS_HRI.ReturnCode_t.ERROR.value
                self.command_result[command_id] = "Bind({}):This_component_is_not_registered".format(component_ref)
        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            self.command_result[command_id] = "Bind({}):Engine_status_is_false".format(component_ref)

        if status == RoIS_HRI.ReturnCode_t.OK.value:
            self.logger.info("{}:{}".format(command_id,self.command_result[command_id]))
        else:
            self.logger.error("{}:{}".format(command_id,self.command_result[command_id]))
        return status

    def bind_any(self, condition):
        if self._engine.state is True:
            status = RoIS_HRI.ReturnCode_t.OK.value
            component_ref_list = ["None"]
        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            component_ref_list = [" "]
        return (status, component_ref_list)

    def release(self, component_ref):
        command_id = self.create_new_id("command")
        if self._engine.state is True:
            if component_ref in self.component_list:
                # component_refからサブHRIエンジンを判別し，そのエンジンのreleaseを実行する
                m = component_ref.split(':')
                engine_name = m[5]
                # サブエンジンの場合
                if engine_name in self.sub_engine.keys():
                    try:
                        s = self.sub_engine[engine_name].release(component_ref)
                        status = s.value
                        self.command_result[command_id] = "Release({}):Status_of_release_in_sub_engine_is_{}".format(component_ref,s)
                    except:
                        status = RoIS_HRI.ReturnCode_t.ERROR.value
                        self.command_result[command_id] = "Release({}):Cannot_connect_to_the_release_method_in_[{}]_engine".format(component_ref,m[5])
                # メインエンジン(自分自身)の場合
                elif engine_name == self.my_engine_name:
                    if component_ref in self.binding:
                        self.binding.remove(component_ref)
                        status = RoIS_HRI.ReturnCode_t.OK.value
                        self.command_result[command_id] = "Release({}):Success"
                    else:
                        status = RoIS_HRI.ReturnCode_t.ERROR.value
                        self.command_result[command_id] = "Release({}):This_component_has_not_been_Bound".format(component_ref)
                # エンジンがない場合
                else:
                    status = RoIS_HRI.ReturnCode_t.ERROR.value
                    self.command_result[command_id] = "Release({}):Engine[{}]_is_not_registered".format(component_ref,m[5])
            else:
                status = RoIS_HRI.ReturnCode_t.ERROR.value
                self.command_result[command_id] = "Release({}):This_component_is_not_registered".format(component_ref)
        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            self.command_result[command_id] = "Release({}):Engine_status_is_false".format(component_ref)
        
        if status == RoIS_HRI.ReturnCode_t.OK.value:
            self.logger.info("{}:{}".format(command_id,self.command_result[command_id]))
        else:
            self.logger.error("{}:{}".format(command_id,self.command_result[command_id]))
        return status

    def get_parameter(self, component_ref, command_id=None):
        if command_id is None:
            command_id = self.create_new_id("command")
        if self._engine.state is True:
            if component_ref in self.component_list:
                # component_refからサブHRIエンジンを判別し，そのエンジンのget_parameterを実行する
                m = component_ref.split(':')
                engine_name = m[5]
                # サブエンジンの場合
                if engine_name in self.sub_engine.keys():
                    try:
                        (s, parameter_list) = self.sub_engine[engine_name].get_parameter(component_ref)
                        status = s.value
                        self.command_result[command_id] = "Get_parameter({}):Status_of_get_parameter_in_sub_engine_is_{}".format(component_ref,s)
                    except:
                        status = RoIS_HRI.ReturnCode_t.ERROR.value
                        self.command_result[command_id] = "Get_parameter({}):Cannot_connect_to_the_get_parameter_method_in_[{}]_engine".format(component_ref,m[5])
                # メインエンジン(自分自身)の場合
                elif engine_name == self.my_engine_name:
                    if component_ref in self.binding:
                        m = component_ref.split(':')
                        comp_name = m[6]
                        comp_param = ['SystemInformation','PersonLocalization','FaceLocalization','SoundLocalization','SpeechRecognition','GestureRecognition',
                                    'SpeechSynthesis','Reaction','Navigation','Follow','Move']
                        if comp_name in comp_param:
                            # comp_urn = m[5] +":"+ m[6]
                            # if comp_urn in self.component_client.keys():
                            if comp_name in self.component_client.keys():
                                try:
                                    if comp_name == 'SystemInformation':
                                        (status, e_status, operable_time) = self.component_client['SystemInformation'].engine_status()
                                        (status, timestamp, robot_ref, position_data) = self.component_client['SystemInformation'].robot_position()
                                        parameters = (status, e_status.value, operable_time, timestamp, robot_ref, position_data)
                                    else:
                                        # parameters = self.component_client[comp_urn].get_parameter()
                                        parameters = self.component_client[comp_name].get_parameter()
                                        
                                    if parameters[0] == RoIS_HRI.ReturnCode_t.OK:
                                        parameter_list = parameters[1:]
                                        status = RoIS_HRI.ReturnCode_t.OK.value
                                        self.command_result[command_id] = 'Get_parameter({}):Parameters_{}'.format(component_ref,str(parameter_list))
                                    else:
                                        status = parameters[0].value
                                        self.command_result[command_id] = "Get_parameter({}):Component_status_is_{}".format(component_ref,parameters[0])
                                except Exception as e:
                                    status = RoIS_HRI.ReturnCode_t.ERROR.value
                                    self.command_result[command_id] = "Get_parameter({}):{}".format(component_ref,e)
                            else:
                                status = RoIS_HRI.ReturnCode_t.ERROR.value
                                self.command_result[command_id] = "Get_parameter({}):Not_found_the_specified_component".format(component_ref)
                        else:
                            status = RoIS_HRI.ReturnCode_t.ERROR.value
                            self.command_result[command_id] = "Get_parameter({}):This_component_doesn't_have_parameter".format(component_ref)
                    else:
                        status = RoIS_HRI.ReturnCode_t.ERROR.value
                        self.command_result[command_id] = "Get_parameter({}):This_component_have_not_been_bound".format(component_ref)
                # エンジンがない場合
                else:
                    status = RoIS_HRI.ReturnCode_t.ERROR.value
                    self.command_result[command_id] = "Get_parameter({}):Engine[{}]_is_not_registered".format(component_ref,m[5])
            else:
                status = RoIS_HRI.ReturnCode_t.ERROR.value
                self.command_result[command_id] = "Get_parameter({}):This_component_is_not_registered".format(component_ref)
        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            self.command_result[command_id] = "Get_parameter({}):Engine_status_is_false".format(component_ref)
        
        if status == RoIS_HRI.ReturnCode_t.OK.value:
            self.logger.info("{}:{}".format(command_id,self.command_result[command_id]))
        else:
            parameter_list = [""]
            self.logger.error("{}:{}".format(command_id,self.command_result[command_id]))
        return (status, parameter_list)

    def set_parameter(self, component_ref, parameters, command_id=None):
        setparam_command_id = command_id
        if setparam_command_id is None:
            command_id = self.create_new_id("command")
        para_num = {'PersonLocalization':2,'FaceLocalization':2,'SoundLocalization':2,'SpeechRecognition':3,
                    'SpeechSynthesis':5,'Reaction':1,'Navigation':3,'Follow':3,'Move':3}
        if self._engine.state is True:
            if component_ref in self.component_list:
                # component_refからサブHRIエンジンを判別し，そのエンジンのset_parameterを実行する
                m = component_ref.split(':')
                engine_name = m[5]
                # サブエンジンの場合
                if engine_name in self.sub_engine.keys():
                    try:
                        (s, command_id) = self.sub_engine[engine_name].set_parameter(component_ref, parameters)
                        status = s.value
                        self.command_result[command_id] = "Set_parameter({}):Status_of_set_parameter_in_sub_engine_is_{}".format(component_ref,s)
                    except:
                        status = RoIS_HRI.ReturnCode_t.ERROR.value
                        self.command_result[command_id] = "Set_parameter({}):Cannot_connect_to_the_set_parameter_method_in_[{}]_engine".format(component_ref,m[5])
                # メインエンジン(自分自身)の場合
                elif engine_name == self.my_engine_name:
                    if component_ref in self.binding:
                        comp_name = m[6]
                        if comp_name in para_num.keys():
                            # comp_urn = m[5] +":"+ m[6]
                            # if comp_urn in self.component_client.keys():
                            if comp_name in self.component_client.keys():
                                if len(parameters) == para_num[comp_name]:
                                    try:
                                        # comp_state = self.component_client[comp_urn].set_parameter(*parameters)
                                        comp_state = self.component_client[comp_name].set_parameter(*parameters)
                                        if comp_state == RoIS_HRI.ReturnCode_t.OK:
                                            # t = threading.Thread(target=self.analyze_component_status, daemon=True, args=(self.component_client[comp_urn],self.command_id,))
                                            t = threading.Thread(target=self.analyze_component_status, daemon=True, args=(self.component_client[comp_name],command_id,))
                                            t.start()
                                            self.command_result[command_id] = "Set_parameter({})".format(component_ref)
                                            status = RoIS_HRI.ReturnCode_t.OK.value
                                        else:
                                            self.command_result[command_id] = "Set_parameter({}):Component_status_is_{}".format(component_ref,comp_state)
                                            status = comp_state.value
                                    except Exception as e:
                                        self.command_result[command_id] = "Set_parameter({}):{}".format(component_ref,e)
                                        status = RoIS_HRI.ReturnCode_t.ERROR.value
                                else:
                                    self.command_result[command_id] = "Set_parameter({}):Incorrect_number_of_arguments;now[{}]/correct[{}]".format(component_ref,str(len(parameters)),str(para_num[comp_name]))
                                    status = RoIS_HRI.ReturnCode_t.ERROR.value
                            else:
                                self.command_result[command_id] = "Set_parameter({}):Not_found_the_specified_component".format(component_ref)
                                status = RoIS_HRI.ReturnCode_t.ERROR.value
                        else:
                            self.command_result[command_id] = "Set_parameter({}):This_component_doesn't_have_set_parameter".format(component_ref)
                            status = RoIS_HRI.ReturnCode_t.ERROR.value
                    else:
                        self.command_result[command_id] = "Set_parameter({}):This_component_have_not_been_bound".format(component_ref)
                        status = RoIS_HRI.ReturnCode_t.ERROR.value
                # エンジンがない場合
                else:
                    status = RoIS_HRI.ReturnCode_t.ERROR.value
                    self.command_result[command_id] = "Set_parameter({}):Engine[{}]_is_not_registered".format(component_ref,m[5])
            else:
                status = RoIS_HRI.ReturnCode_t.ERROR.value
                self.command_result[command_id] = "Set_parameter({}):This_component_is_not_registered".format(component_ref)
        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            self.command_result[command_id] = "Set_parameter({}):Engine_status_is_false".format(component_ref)
        
        if status == RoIS_HRI.ReturnCode_t.OK.value:
            self.logger.info("{}:{}".format(command_id,self.command_result[command_id]))
        else:
            self.logger.error("{}:{}".format(command_id,self.command_result[command_id]))
        return (status, command_id)

    def execute(self, command_unit_list, is_branch=False, command_id_base=None): # command_unit_list: command_message.CommandUnitSequence
        self.logger.info("Start Execute")
        # self.logger.info(command_unit_list)
        execute_id = self.create_new_id("execute",is_branch=is_branch,command_id_base=command_id_base)
        if self._engine.state is True:
            # self.logger.info("OK")
            status = RoIS_HRI.ReturnCode_t.OK.value
            try:
                d = json.loads(command_unit_list)
                # self.logger.info(d)
                if is_branch == False:
                    if self.check_branch(d,execute_id) == False: 
                        self.command_result[execute_id] = "Execute({}):Can't access the set_parameter in the same component from different branches".format(execute_id)
                        self.logger.error(self.command_result[execute_id])
                        status = RoIS_HRI.ReturnCode_t.ERROR.value
                        return (status, command_unit_list)
                    command_list = d["command_unit_list"]
                else:
                    command_list = d["command_list"]
                i = 0
                for c in command_list:
                    # self.logger.info(c)
                    if "branch_list" not in c.keys():
                        i += 1
                        # self.logger.info(c_k[0])
                        command_id = self.create_new_id("execute",command_id_base=command_id_base,command_num=i)
                        component_ref = c["component_ref"]
                        command_type = c["command_type"]  
                        c["command_id"] = command_id

                        # self.logger.info(c["arguments"]["parameters"])
                        arguments = {p["name"]:p["value"] for p in list(c["arguments"]["parameters"])}
                        delay_time = c["delay_time"]
                        # self.logger.info("component_ref:{},command_type:{},arguments:{},delay_time:{}".format(component_ref,command_type,arguments,delay_time))
                        # self.logger.info(d)
                        
                        if component_ref in self.component_list:
                            time.sleep(delay_time * 0.001)
                            if command_type == "set_parameter":
                                (status, c_id) = self.set_parameter(component_ref,list(arguments.values()),command_id)
                                self.command_result[command_id] = "Execute({},{}):Finished set parameter / set_parameter id = {}".format(component_ref,command_type,c_id)
                                self.logger.info(self.command_result[command_id])
                                c["command_id"] = c_id
                                if status == RoIS_HRI.ReturnCode_t.ERROR.value:
                                    self.command_result[execute_id] = "Execute({},{}):Status of Set_Parameter is ERROR".format(component_ref,command_type)
                                    self.logger.error(self.command_result[execute_id])
                                    command_unit_list = json.dumps(d, indent=4) 
                                    return (status,command_unit_list)
                            elif command_type == "get_parameter":
                                (status, parameter_list) = self.get_parameter(component_ref)
                                self.command_result[command_id] = "Execute({},{}):Finished get parameter".format(component_ref,command_type)
                                self.logger.info(self.command_result[command_id])
                                # サブエンジンのget_parameterのidは取得できない
                                if status == RoIS_HRI.ReturnCode_t.ERROR.value:
                                    self.command_result[execute_id] = "Execute({},{}):Status of Get_Parameter is ERROR".format(component_ref,command_type)
                                    self.logger.error(self.command_result[execute_id])
                                    command_unit_list = json.dumps(d, indent=4)
                                    return (status,command_unit_list)
                            else:
                                status = RoIS_HRI.ReturnCode_t.ERROR.value
                                self.command_result[execute_id] = "Execute({},{}):The command_type is undefined".format(component_ref,command_type)
                                self.logger.error(self.command_result[execute_id])
                                command_unit_list = json.dumps(d, indent=4)
                                return (status,command_unit_list)
                        else:
                            status = RoIS_HRI.ReturnCode_t.ERROR.value
                            self.command_result[execute_id] = "Execute({},{}):This_component_is_not_registered".format(component_ref,command_type)
                            self.logger.error(self.command_result[execute_id])
                            command_unit_list = json.dumps(d, indent=4)
                            return (status,command_unit_list)

                    else: # ConcurrentCommands(Branch list) -> Branch(CommandMessage list)
                        branch_delay_time = c["delay_time"]
                        time.sleep(branch_delay_time * 0.001)
                        futures = []
                        results = []
                        execute_branch_num = 1
                        for branch in c["branch_list"]:
                            if command_id_base is None:
                                id_base = "b-{}".format(execute_branch_num)
                            else:
                                id_base = "{}-b-{}".format(command_id_base, execute_branch_num)
                            cul = json.dumps(branch, indent=4)
                            executor = ThreadPoolExecutor(max_workers=None)
                            future = executor.submit(self.execute, cul, is_branch=True, command_id_base=id_base)
                            futures.append(future)
                            execute_branch_num += 1
                        for future in futures:
                            results.append(future.result())
                        executor.shutdown()
                        for x in range(len(c["branch_list"])):
                            j = json.loads(results[x][1])
                            c["branch_list"][x] = j
                    
                command_unit_list = json.dumps(d, indent=4)    
                self.command_result[execute_id] = "Execute({}): Success".format(execute_id)
                self.logger.info(self.command_result[execute_id])

            except Exception as e:
                status = RoIS_HRI.ReturnCode_t.ERROR.value
                self.command_result[execute_id] = "Execute:{}".format(e)
                self.logger.error(self.command_result[execute_id])
                command_unit_list = json.dumps(d, indent=4)
                return (status,command_unit_list)

        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            self.command_result[execute_id] = "Execute:Engine_status_is_false"
            self.logger.error(self.command_result[execute_id])
            return (status, command_unit_list)
        
        status = RoIS_HRI.ReturnCode_t.OK.value
        if is_branch == True:
            self.logger.info("Execute:this is branch")
        else:
            self.logger.info("status:{},command_unit_list:{}".format(status,command_unit_list))
            # self.logger.info("command_ids: {}".format(self.command_result))
        return (status, command_unit_list)

    def get_command_result(self, command_id, condition):
        if self._engine.state is True:
            # command_idからサブHRIエンジンを判別し，そのエンジンのget_command_resultを実行する
            e = command_id.split('-')
            engine_name = e[0]
            # サブエンジンの場合
            if engine_name in self.sub_engine.keys():
                try:
                    (s, results) = self.sub_engine[engine_name].get_command_result(command_id, condition)
                    status = s.value
                    if s == RoIS_HRI.ReturnCode_t.OK:
                        self.logger.info("Get_Command_Result({}):Status_of_get_command_result_in_sub_engine_is_{}".format(command_id,s))
                    else:
                        self.logger.error("Get_Command_Result({}):Status_of_get_command_result_in_sub_engine_is_{}".format(command_id,s))
                except:
                    status = RoIS_HRI.ReturnCode_t.ERROR.value
                    results = ["Cannot_connect_to_the_engine[{}]".format(e[0])]
                    self.logger.error("Get_Command_Result({}):Cannot_connect_to_the_get_command_result_in_[{}]_engine".format(command_id,e[0]))
            # メインエンジン(自分自身)の場合
            elif engine_name == self.my_engine_name:
                if command_id in self.command_result.keys():
                    status = RoIS_HRI.ReturnCode_t.OK.value
                    results = [self.command_result[command_id]]
                    self.logger.info("Get_Command_Result({}):Success <results:{}>".format(command_id,results))
                else:
                    status = RoIS_HRI.ReturnCode_t.ERROR.value
                    results = ["Command_id({})_is_wrong".format(command_id)]
                    self.logger.error("Get_Command_Result({}):Command_id_is_wrong".format(command_id))
            else:
                status = RoIS_HRI.ReturnCode_t.ERROR.value
                results = ["Engine[{}]_is_not_registered".format(e[0])]
                self.logger.error("Get_Command_Result({}):Engine[{}]_is_not_registered".format(command_id,e[0]))
        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            results = ["Engine_state_is_false"]
            self.logger.error("Get_Command_Result({}):Engine_status_is_false".format(command_id))
        return (status, results)


class QueryIF(RoIS_HRI.QueryIF):
    """
    class QueryIF(object):
    """

    def __init__(self, Engine):
        self._engine = Engine

    def query(self, query_type, condition):
        if self._engine.state is True:
            # サブHRIエンジンを呼ぶ必要はない
            status = RoIS_HRI.ReturnCode_t.OK.value
            profile = message_profile.Analyze_Component_Profile(query_type)
            self.component_profile = profile.get_profile()
            results = self.component_profile
            self.logger.info("Query({}):Success:{}:{}".format(query_type,status,results))
        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            results = [" "]
            self.logger.error("Query({}):Engine_status_is_false".format(query_type))
        return (status, results)


class EventIF(RoIS_HRI.EventIF):
    """
    class QueryIF(object):
    """

    def __init__(self, Engine):
        self._engine = Engine

    def subscribe(self, event_type, condition):
        if self._engine.state is True:
            # subscribe_id = "{}-{}".format(self.my_engine_name, self.subscribe_id_num) 
            # # サブエンジンのsubscribeも同時に行うためidは全て共通なものにする
            subscribe_id = self.create_new_id("subscribe")
            self.subscribed_event[subscribe_id] = event_type
            try:
                for x in self.sub_engine.keys():
                    self.sub_engine[x].subscribe(event_type, condition)
                status = RoIS_HRI.ReturnCode_t.OK.value
                self.logger.info("Subscribe({}):Success <subscribed_event:{}>".format(event_type, self.subscribed_event))
            except:
                status = RoIS_HRI.ReturnCode_t.ERROR.value
                self.logger.error("Subscribe({}):Cannot_run_Subscribe_in_sub_engine".format(event_type))
                # subscribe_id = "{}-00".format(self.my_engine_name)
                subscribe_id = "00"
        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            # subscribe_id = "{}-00".format(self.my_engine_name)
            subscribe_id = "00"
            self.logger.error("Subscribe({}):Engine_status_is_false <subscribed_event:{}>".format(event_type,self.subscribed_event))
        return (status, subscribe_id)

    def unsubscribe(self, subscribe_id):
        if self._engine.state is True:
            if subscribe_id in self.subscribed_event.keys():
                del self.subscribed_event[subscribe_id]
                try:
                    for x in self.sub_engine.keys():
                        self.sub_engine[x].unsubscribe(subscribe_id)
                    status = RoIS_HRI.ReturnCode_t.OK.value
                    self.logger.info("Unsubscribe({}):Success <subscribed_event:{}>".format(subscribe_id,self.subscribed_event))
                except:
                    status = RoIS_HRI.ReturnCode_t.ERROR.value
                    self.logger.error("Unsubscribe({}):Cannot_run_Unsubscribe_in_sub_engine".format(subscribe_id))
            else:
                status = RoIS_HRI.ReturnCode_t.ERROR.value
                self.logger.error("Unsubscribe({}):Subscribe_id_is_wrong(subscribed_event:{})".format(subscribe_id,self.subscribed_event))
        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            self.logger.error("Unsubscribe({}):Engine_status_is_false(subscribed_event:{})".format(subscribe_id,self.subscribed_event))
        return status

    def get_event_detail(self, event_id, condition):
        if self._engine.state is True:
            e = event_id.split('-')
            engine_name = e[0]
            # サブエンジンの場合
            if engine_name in self.sub_engine.keys():
                try:
                    (s, results) = self.sub_engine[engine_name].get_event_detail(event_id, condition)
                    status = s.value
                    if s == RoIS_HRI.ReturnCode_t.OK:
                        self.logger.info("Get_Event_Detail({}):Status_of_Get_Event_Detail_in_sub_engine_is_{}".format(event_id,s))
                    else:
                        self.logger.error("Get_Event_Detail({}):Status_of_Get_Event_Detail_in_sub_engine_is_{}".format(event_id,s))
                except:
                    status = RoIS_HRI.ReturnCode_t.ERROR.value
                    results = ["Cannot_connect_to_the_engine[{}]".format(e[0])]
                    self.logger.error("Get_Event_Detail({}):Cannot_connect_to_the_Get_Event_Detail_in_[{}]_engine".format(event_id,e[0]))
            # メインエンジン(自分自身)の場合
            elif engine_name == self.my_engine_name:
                if event_id in self.event_detail.keys():
                    status = RoIS_HRI.ReturnCode_t.OK.value
                    results = self.event_detail[event_id]
                    self.logger.info("Get_Event_Detail({}):Success <results:{}>".format(event_id,results))
                else:
                    status = RoIS_HRI.ReturnCode_t.ERROR.value
                    results = ["Event_id({})_is_wrong".format(event_id)]
                    self.logger.error("Get_Event_Detail({}):Event_id_is_wrong".format(event_id))
            else:
                status = RoIS_HRI.ReturnCode_t.ERROR.value
                results = ["Engine[{}]_is_not_registered".format(e[0])]
                self.logger.error("Get_Event_Detail({}):Engine[{}]_is_not_registered".format(event_id,e[0]))
        else:
            status = RoIS_HRI.ReturnCode_t.ERROR.value
            results = ["Engine_state_is_false"]
            self.logger.error("Get_Event_Detail({}):Engine_status_is_false".format(event_id))
        return (status, results)


class IF(SystemIF, CommandIF, QueryIF, EventIF, Service_Application_Base_sample.Service_Application_Base):
    """IF
    """
    def __init__(self, Engine, engine_profile_file, client_info, my_engine_urn=None, logger=None):
        self._engine = Engine

        p = message_profile.Analyze_Engine_Profile(engine_profile_file, my_engine_urn)
        (self.engine_profile, self.engine_and_component ,self.component_list) = p.analysis()
        self.engine_list = list(self.engine_and_component.keys())
        m = self.engine_list[0].split(':')
        self.my_engine_name = m[6]
        self.init_logger(self.my_engine_name,logger)
        self.init_vars()
        self.init_instance(client_info)
        
        Service_Application_Base_sample.Service_Application_Base.__init__(self,logger=self.logger)

        # sub_engine connection_test
        for x in self.sub_engine.keys():
            for i in range(3):
                try:
                    c = self.sub_engine[x].connect()
                    self.logger.info("Init: Connect_to_[{}]".format(x))
                    self.logger.info("Init: Result_of_connect".format(c))
                except Exception as e1:
                    if i != 2:
                        self.logger.error("Init: {}".format(str(e1)))
                        self.logger.error("Init: Try_again_to_connect_to_[{}]".format(x))
                        time.sleep(1)
                    else:
                        self.logger.error("Init: Cannot_connect_to_[{}]".format(x))
                        sys.exit()
                else:
                    break
            
        # component connection_test
        for y in self.component_client.keys():
            for j in range(3):
                try:
                    if y == 'SystemInformation':
                        (status, e_status, operable_time) = self.component_client[y].engine_status()
                        self.logger.info("Init: Result_of_component_status [{}]".format(e_status))
                    else:
                        cs = self.component_client[y].component_status()
                        self.logger.info("Init: Result_of_component_status [{}]".format(cs[1]))
                    # cs = self.component_client[y].component_status()
                    # self.logger.info("Init: Result_of_component_status [{}]".format(cs[1]))
                    self.logger.info("Init: Connect_to_[{}]".format(y))
                except Exception as e2:
                    if j != 2:
                        self.logger.error("Init: {}".format(str(e2)))
                        self.logger.error("Init: Try_again_to_connect_to_[{}]".format(y))
                        time.sleep(1)
                    else:
                        self.logger.error("Init: Cannot_connect_to_[{}]".format(y))
                        sys.exit()
                else:
                    break
        
        self.lock = threading.Lock()
        self.th_mon = threading.Thread(target=self.monitor_event, daemon=True)
        self.th_mon.start()

        if len(self.sub_engine) > 0:
            self.th_mon_sub = threading.Thread(target=self.monitor_event_from_sub_engine, daemon=True)        
            self.th_mon_sub.start()
        else:
            pass

        self.logger.info("main_engine_server: IF init completed")
    
    def init_instance(self, client_info):
        # 2層以内のエンジンの階層構造を想定している
        self.main_event_q = queue.Queue() # check event from main engine
        self.sub_event_q = queue.Queue() # check event from sub engine
        self.sub_engine = {}
        self.component_client = {}
        event_occur_component = ['PersonDetection','PersonLocalization','PersonIdentification','FaceDetection','FaceLocalization',
                                'SoundDetection','SoundLocalization','SpeechRecognition','GestureRecognition','VirtualNavigation']

        with open("{}.json".format(client_info),"r") as f:
            d = json.load(f)
            # print(d)
            try:
                for x in self.engine_list[1:]:
                    # print(x)
                    s = x.split(':')
                    s_name = s[6]
                    s_module = importlib.import_module("pyrois_common.{}".format(str(d[x]["module"])))
                    s_method = getattr(s_module, d[x]["class"])
                    self.sub_engine[s_name] = s_method(d[x]["url"],logger=self.logger_client,event_queue=self.sub_event_q)
                self.logger.info("self.sub_engine: {}".format(str(self.sub_engine)))
                for y in self.engine_and_component[self.engine_list[0]]:
                    # print(x)
                    c = y.split(':')
                    c_name = c[6]
                    c_module = importlib.import_module("pyrois_common.{}".format(d[y]["module"])) # PYTHONPATHでコンポーネントのディレクトリ指定　export PYTHONPATH="$(pwd):$PYTHONPATH"
                    # c_module = importlib.import_module("{}".format(d[y]["module"])) 
                    c_method = getattr(c_module, d[y]["class"])
                    if c_name in event_occur_component:
                        self.component_client[c_name] = c_method(d[y]["url"],self.main_event_q)
                    else:
                        self.component_client[c_name] = c_method(d[y]["url"])
                self.logger.info("self.component_client: {}".format(str(self.component_client)))
            except Exception as e:
                self.logger.error("Init_instance(HRI_Component): {}".format(str(e)))
                sys.exit()       

    def init_logger(self, engine_name, logger):
        if logger is not None:
            self.logger = logger
        else:
            self.dt = datetime.now()
            self.dt_format = self.dt.strftime("%Y%m%d-%H%M")
            self.os = platform.system()
            self.logger = logging.getLogger('[{}]_Engine: HRI_Engine_sample.py'.format(engine_name))
            self.logger.setLevel(logging.DEBUG)
            formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
            self.fh = logging.FileHandler('{}.log'.format(self.dt_format))
            self.fh.setLevel(logging.DEBUG)
            self.fh.setFormatter(formatter)
            self.logger.addHandler(self.fh)

            self.ch = logging.StreamHandler()
            self.ch.setLevel(logging.ERROR)
            self.ch.setFormatter(formatter)
            self.logger.addHandler(self.ch)
        
        self.logger_client = logging.getLogger('[{}]_Engine: HRI_Engine_client_sample.py'.format(engine_name))
        self.logger_client.setLevel(logging.DEBUG)
        self.logger_client.addHandler(self.fh)
        self.logger_client.addHandler(self.ch)

        self.logger.debug("[{}]_Server running".format(engine_name))
        self.logger.info("This_logger_has_also_logs_in_Service_Application_Base_sample.py")
    
    def init_vars(self):
        self.command_id_num = 0
        self.command_result = {}
        self.subscribe_id_num = 0
        self.subscribed_event = {}
        self.error_id_num = 0
        self.error_detail = {}
        self.event_id_num = 0
        self.binding = []
        self.component_profile = []
        self.event_detail = {}
        self.execute_id_num = 0

    def analyze_component_status(self, component, command_id):
        d = deque([],2)
        error_id = self.create_new_id("error")
        e = [RoIS_Common.Component_Status.BUSY, RoIS_Common.Component_Status.READY]
        while True:
            # 現状ではどのコンポーネントの実行が終了したか分からない．
            with self.lock:
                (_, c_status) = component.component_status()
                d.append(c_status)
            # print(d)

            if c_status == RoIS_Common.Component_Status.ERROR:
                with self.lock:
                    # self.notify_error(error_id, RoIS_Service.ErrorType.COMPONENT_INTERNEL_ERROR.value)
                    # self.logger.info("analyze_component_status:notify_error")
                    # self.error_detail[error_id] = "component_status_is_error"
                    self.completed(command_id, RoIS_Service.Completed_Status.ERROR.value)
                    self.logger.error("analyze_component_status:completed error")
                    
                return

            if len(d) and list(d) == e:
                with self.lock:
                    self.completed(command_id, RoIS_Service.Completed_Status.OK.value)
                    self.logger.info("analyze_component_status:completed")
                return
            time.sleep(0.5)

    def monitor_event(self):
        self.logger.info("monitor_event:start")
        while True:
            x = self.main_event_q.get()
            # print(x)
            event_type = x[0]

            event_id = self.create_new_id("event")
            #self.logger.info("monitor_event:eventtype({}),subscribed_event({})".format(event_type, self.subscribed_event))

            if event_type in self.subscribed_event.values():
                subscribe_id = [k for k, v in self.subscribed_event.items() if v == event_type ]
                for idx in subscribe_id:
                    self.notify_event(event_id, event_type, idx, datetime.now().isoformat())
                self.event_detail[event_id] = x
                self.logger.debug("event_detail:{}".format(x))
                self.logger.info("monitor_event:notify_event(subscribe_id:{})".format(subscribe_id))
            else:
                self.logger.info("monitor_event:This_event({})_is_not_subscribed".format(event_type))

    def monitor_event_from_sub_engine(self):
        while True:
            (params, methodname) = self.sub_event_q.get()
            if methodname == 'completed':
                status = params[1].value
                self.completed(params[0], status)
            elif methodname == 'notify_error':
                error_type = params[1].value
                self.notify_error(params[0], error_type)
            elif methodname == 'notify_event':
                self.notify_event(params[0], params[1], params[2], params[3])
            else:
                print("methodname={}".format(methodname))
                pass
    
    def create_new_id(self, id_type, command_id_base=None, command_num=None, is_branch=False):
        with self.lock:
            if id_type == "command":
                self.command_id_num += 1
                command_id = "{}-{}".format(self.my_engine_name, self.command_id_num)
                return command_id
            elif id_type == "execute":
                if command_num == None:
                    if is_branch == False:
                        self.execute_id_num += 1    
                    command_id = "{}-exe-{}".format(self.my_engine_name, self.execute_id_num)
                else:
                    if command_id_base == None:
                        command_id = "{}-exe-{}-{}".format(self.my_engine_name, self.execute_id_num, command_num)
                    else:
                        command_id = "{}-exe-{}-{}-{}".format(self.my_engine_name, self.execute_id_num, command_id_base, command_num)
                return command_id
            elif id_type == "subscribe":
                self.subscribe_id_num += 1
                subscribe_id = str(self.subscribe_id_num)
                return subscribe_id
            elif id_type == "event":
                self.event_id_num += 1
                event_id = "{}-{}".format(self.my_engine_name, self.event_id_num)
                return event_id
            elif id_type == "error":
                self.error_id_num += 1
                error_id = "{}-{}".format(self.my_engine_name, self.error_id_num)
                return error_id
            else:
                return False
        
    def check_branch(self, command_unit_list_dict, execute_id):
        branch_list_num = 0
        is_correct = True
        command_list = command_unit_list_dict["command_unit_list"]
        for command in command_list:
            if "branch_list" in command.keys():
                branch_list_num += 1
                all_branch_comprefs = []
                branch_num = 0
                all_branch_comprefs = {}
                for branch in command["branch_list"]:
                    branch_num += 1
                    branch_command_list = branch["command_list"]
                    component_refs = [branch_command["component_ref"] for branch_command in branch_command_list if branch_command["command_type"] == "set_parameter"]
                    all_branch_comprefs[branch_num] = component_refs
                for combi in itertools.combinations(all_branch_comprefs.keys(),2):
                    common = set(all_branch_comprefs[combi[0]]).intersection(set(all_branch_comprefs[combi[1]]))
                    if len(common) > 0:
                        is_correct = False
                        self.logger.error("Execute({}) [BranchList_num:{}, Branch_Combination:{}, Same_Components:{}]: Cannot access the set_parameter in the same component from different branches.".format(execute_id,branch_list_num,combi,common))
        if is_correct == False:
            return False
        else:
            return True


class IF_server:
    """IF_Server
    """
    def __init__(self, port):
        self._addr = os.getenv("HRIENGINE")
        self._server = ThreadingXMLRPCServer(
            ("0.0.0.0", port), logRequests=False)

    def run(self, _IF):
        """IF_Server
        """
        self._server.register_instance(_IF)
        self._server.register_introspection_functions()
        # print("server running")
        self._server.serve_forever()
        # print("server finished")


class MyHRIE:
    """IF_Server
    """
    def __init__(self):
        self.state = False


def test_engine(port, engine_profile_file=None, client_info=None, my_engine_urn=None, logger=None):
    """test_engine
    """
    IF_server(port).run(IF(MyHRIE(), engine_profile_file, client_info, my_engine_urn, logger))

if __name__ == "__main__":
    # コマンドラインでの実行の場合はloggerの指定は厳しい
    args = sys.argv
    if len(args) == 6:
        print(args)
        args[1] = int(args[1])
        for i in range(len(args)):
            if args[i] == "None":
                args[i] = None
            else:
                pass
        print(args)
        test_engine(*args[1:])
        del sys.argv[1:]
    else:
        print("The number of arguments is wrong. The argument types are [port, engine_profile_file, client_info, my_engine_urn, logger].")
        sys.exit()



# export PYTHONPATH="$(pwd)/pyRoIS_higashi:$PYTHONPATH"
# python3 HRI_Engine_sample.py 8000 engine_profile_gazebo client_info_gazebo None None   
# ^ pyRoIS_higashiディレクトリに移動しないと実行できないが、移動すると実行できなくなる 親ディレクトリから子ディレクトリの実行はできないか

# python3 -c "from pyRoIS_higashi import HRI_Engine_sample; HRI_Engine_sample.test_engine(8000,"engine_profile_gazebo", "client_info_gazebo", None, None)"
# ^ もう一つの手段 ただし、実行結果は上記方法と同様  pyRoIS_higashiディレクトリ内での実行になってしまう


# unittest_main_sub_engine3.py
#
# Copyright 2020 Ryota Higashi
# This software may be modified and distributed under the terms
# of the MIT license
#
# for python 3

"""unittest サブエンジンが存在しない場合のテスト
"""

import time
from multiprocessing import Process, Queue
import unittest
import xmlrpc.client
import sys
import re
import logging
import platform
from datetime import datetime

from pyrois import RoIS_HRI, RoIS_Common, RoIS_Service
import HRI_Engine_sample, HRI_Engine_client_sample
import command_message

from component import System_Information
from component import Person_Detection
from component import Person_Localization
from component import Person_Identification
from component import Face_Detection
from component import Face_Localization
from component import Sound_Detection
from component import Sound_Localization
from component import Speech_Recognition
from component import Gesture_Recognition
from component import Speech_Synthesis
from component import Reaction
from component import Navigation
from component import Follow
from component import Move


eng_profile = """<?xml version="1.0" encoding="UTF-8"?>
<!-->
Robotic Interaction Service Framework Version 1.2
    > Annex A: Examples of Profile in XML
        > A.4 HRI Engine Profile (p.102 (PDF:p.115)) reference
<-->

<rois:HRIEngineProfile gml:id="engine_profile"
    xmlns:rois="http://www.omg.org/spec/RoIS/20151201"
    xmlns:gml="http://www.opengis.net/gml/3.2">
    <gml:identifier>urn:x-rois:def:HRIEngine:Kyutech::main</gml:identifier>
    <gml:name>main</gml:name>
    <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization</rois:HRIComponent>
</rois:HRIEngineProfile>"""

commnd_unit_list ="""{
    "command_unit_list": [
        {
            "component_ref": "urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization",
            "command_type": "set_parameter",
            "command_id": "main-4",
            "arguments": {
                "parameters": [
                    {
                        "name": "detection_threshold",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": 555
                    },
                    {
                        "name": "minimum_interval",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": 222
                    }
                ]
            },
            "delay_time": 5
        },
        {
            "component_ref": "urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization",
            "command_type": "get_parameter",
            "command_id": "main-5",
            "arguments": {
                "parameters": [
                    {
                        "name": "",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": ""
                    }
                ]
            },
            "delay_time": 3
        },
        {
            "component_ref": "urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization",
            "command_type": "set_parameter",
            "command_id": "main-6",
            "arguments": {
                "parameters": [
                    {
                        "name": "detection_threshold",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": 300
                    },
                    {
                        "name": "minimum_interval",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": 100
                    }
                ]
            },
            "delay_time": 10
        },
        {
            "component_ref": "urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization",
            "command_type": "get_parameter",
            "command_id": "main-7",
            "arguments": {
                "parameters": [
                    {
                        "name": "",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": ""
                    }
                ]
            },
            "delay_time": 2
        }
    ]
}"""

command_unit_list_dict = {'command_unit_list': [{'component_ref': 'urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization', 'command_type': 'set_parameter', 'command_id': 'main-4', 'arguments': {'parameters': [{'name': 'detection_threshold', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 555}, {'name': 'minimum_interval', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 222}]}, 'delay_time': 5}, {'component_ref': 'urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization', 'command_type': 'get_parameter', 'command_id': 'main-5', 'arguments': {'parameters': [{'name': '', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': ''}]}, 'delay_time': 3}, {'component_ref': 'urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization', 'command_type': 'set_parameter', 'command_id': 'main-6', 'arguments': {'parameters': [{'name': 'detection_threshold', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 300}, {'name': 'minimum_interval', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': 100}]}, 'delay_time': 10}, {'component_ref': 'urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization', 'command_type': 'get_parameter', 'command_id': 'main-7', 'arguments': {'parameters': [{'name': '', 'data_type_ref': {'codebook_reference': '', 'version': ''}, 'value': ''}]}, 'delay_time': 2}]}

event_result = {
    (('main-4', RoIS_Service.Completed_Status.OK), 'completed'), # completed set_parameter
    (('main-6', RoIS_Service.Completed_Status.OK), 'completed'), # completed set_parameter
    # (('main-1', 'person_localized', '1'), 'notify_event'),
    # (('main-2', 'person_localized', '1'), 'notify_event'),
    # (('main-3', 'person_detected', '2'), 'notify_event'),
    # (('main-4', 'person_detected', '2'), 'notify_event')
}


class Execute_test(unittest.TestCase):    
    maxDiff = None
    
    def setUp(self):
        """ setUP
        """
        # start the server process
        self.ps = [Process(target=HRI_Engine_sample.test_engine, args=(8016,"engine_profile_3_1", "client_info1", "urn:x-rois:def:HRIEngine:Kyutech::main", None,)),
            # Process(target=Person_Detection.example_pd, args=(8002,)),
            Process(target=Person_Localization.example_pl, args=(8003,))
            # Process(target=System_Information.example_si, args=(8001,))
            ]
        
        for x in reversed(self.ps):
            x.start()
            time.sleep(0.5)

        time.sleep(5.0)
        

    def test_IF(self):
        """ test_IF
        """
        c_ri = command_message.RoIS_Identifier("","")
        c_p1 = command_message.Parameter("detection_threshold", c_ri, 555)
        c_p2 = command_message.Parameter("minimum_interval", c_ri, 222)
        c_al1 = command_message.ArgumentList([c_p1,c_p2])
        c_cm1 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization',"set_parameter","",c_al1)
        c_cm1.delay_time = 5
        
        c_p3 = command_message.Parameter("detection_threshold", c_ri, 300)
        c_p4 = command_message.Parameter("minimum_interval", c_ri, 100)
        c_al2 = command_message.ArgumentList([c_p3,c_p4])
        c_cm2 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization',"set_parameter","",c_al2)
        c_cm2.delay_time = 10
    
        c_p5 = command_message.Parameter("", c_ri, "")
        c_al3 = command_message.ArgumentList([c_p5])
        c_cm3 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization',"get_parameter","",c_al3)
        c_cm3.delay_time = 3

        c_p6 = command_message.Parameter("", c_ri, "")
        c_al4 = command_message.ArgumentList([c_p6])
        c_cm4 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization',"get_parameter","",c_al4)
        c_cm4.delay_time = 2
        
        c_cus = command_message.CommandUnitSequence([c_cm1,c_cm3,c_cm2,c_cm4])


        i = HRI_Engine_client_sample.IF("http://127.0.0.1:8016")
        res = [
            i.connect(),
            i.bind('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization'),
            i.get_parameter('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization'),
            i.execute(c_cus),
            i.get_command_result("main-3","")
        ]
            
        print(res)

        s_e = set()
        for x in range(2):
            e = i.get_event()
            s_e.add((e[0][:3],e[1]))
        
        res.extend([
            i.disconnect()
        ])

        self.assertEqual(s_e,event_result)
        return self.assertEqual(res,
                                [
                                RoIS_HRI.ReturnCode_t.OK,
                            
                                RoIS_HRI.ReturnCode_t.OK, # bind:PersonLocalization

                                (RoIS_HRI.ReturnCode_t.OK,[1000,20]), # get_parameter:PersonLocalization
                                (RoIS_HRI.ReturnCode_t.OK,commnd_unit_list), # execute

                                (RoIS_HRI.ReturnCode_t.OK,["Execute({}): Success".format(command_unit_list_dict)]), # get_command_result
                                
                                RoIS_HRI.ReturnCode_t.OK # disconnect
                                ])


    def tearDown(self):
        """ tearDown
        """
        # terminate the server process
        for x in self.ps:
            if x.is_alive():
                x.terminate()

        time.sleep(1)


if __name__ == '__main__':
    # ip = sys.argv
    # if len(ip) > 1:
    #     m = re.search(r'(\d+)\.(\d+)\.(\d+)\.(\d+)',ip[1])
    # else:
    #     m = None
    # if m and len(m.groups()) == 4:
    #     ser_uri = "http://" + ip[1] +":8000"
    #     print("server_url == " + ser_uri + "\n")
    #     del sys.argv[1]
    # else:
    #     ser_uri = "http://127.0.0.1:8000"
    #     print("server_url == " + ser_uri + "\n")
    #     if len(ip) > 1 and ip[1][0] != '-':
    #         del sys.argv[1]

    unittest.main()


# System_Information_Client.py
#
# Copyright 2019 Ryota Higashi
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python3
# For HRI Engine

"""System_Information_Client
"""

import threading
# import logging
import xmlrpc.client

from pyrois import RoIS_Common, RoIS_HRI


class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, uri):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)

    def robot_position(self):
        (status, timestamp, robot_ref, position_data) = self._proxy.robot_position()
        return(RoIS_HRI.ReturnCode_t(status), timestamp, robot_ref, position_data)

    def engine_status(self):
        (status, e_status, operable_time) = self._proxy.engine_status()
        return (RoIS_HRI.ReturnCode_t(status), RoIS_Common.Component_Status(e_status), operable_time)


class System_Information_Client(Query):
    """System_Information_Client
    """
    def __init__(self, uri):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)


# Sound_Localization.py
#
# Copyright 2019 Ryota Higashi
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python3
# For HRI Component

"""Sound_Localization
"""

import sys
import queue
import time
from datetime import datetime
import threading

from pyrois import RoIS_Common, RoIS_HRI

import socketserver
import xmlrpc.server

class ThreadingXMLRPCServer(socketserver.ThreadingMixIn, xmlrpc.server.SimpleXMLRPCServer):
    """ThreadingXMLRPCServer
    """
    pass


class Command(RoIS_Common.Command):
    """Command
    """
    def __init__(self, c):
        self._component = c

    def start(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def stop(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def suspend(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def resume(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status

    def set_parameter(self, detection_threshold, minimum_interval):
        arg = [detection_threshold, minimum_interval]
        self.parameter_queue.put(arg)
        self.Detection_Threshold = detection_threshold
        self.Minimum_Interval = minimum_interval

        th = threading.Thread(target=self.execute, daemon=True)
        th.start()
        status = RoIS_HRI.ReturnCode_t.OK.value
        return status


class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, c):
        self._component = c
        
    def component_status(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        with self._lock:
            c_status = self.c_status
        return (status, c_status)
    
    def get_parameter(self):
        status = RoIS_HRI.ReturnCode_t.OK.value
        self.c_status = RoIS_Common.Component_Status.READY.value
        return (status, self.Detection_Threshold, self.Minimum_Interval)
    

class Event(RoIS_Common.Event):
    """Event
    """
    def __init__(self, c):
        self._component = c
        self.event_queue = queue.Queue()

    def poll_event(self):
        """poll_event
        """
        msg = self.event_queue.get()
        return msg

    def sound_localized(self, timestamp, sound_ref, position_data):
        """sound_localization
        """
        msg = xmlrpc.client.dumps((timestamp, sound_ref, position_data), 'sound_localized')
        self.event_queue.put(msg)


class Sound_Localization(Event, Command, Query):
    """Sound_Localization
    """
    def __init__(self, c, lock=None):
        super().__init__(c)
        self._component = c
        if lock is None:
            self._lock = threading.Lock()
        else:
            self._lock = lock
        
        self.c_status = RoIS_Common.Component_Status.READY.value
        self.parameter_queue = queue.Queue()
        
        self.Detection_Threshold = 1000
        self.Minimum_Interval = 20

    def execute(self):  # dummy
        """execute component
        """
        arg = self.parameter_queue.get()
        with self._lock:
            self.c_status = RoIS_Common.Component_Status.BUSY.value

        time.sleep(1)

        with self._lock:
            self.c_status = RoIS_Common.Component_Status.READY.value
        

class component:
    """component
    """
    def __init__(self):
        self._state = False


def event_dispatch(sl):
    """event_dispatch
    """
    # sl.sound_localized(datetime.now().isoformat(), [""], [""])
    # time.sleep(0.1)
    # sl.sound_localized(datetime.now().isoformat(), [""], [""])


def example_sl(port):
    """example_sl
    """
    c = component()
    sl = Sound_Localization(c)

    # start the timer to dispatch events
    t = threading.Timer(0.1, event_dispatch, args=(sl,))
    t.start()

    # start the XML-RPC server
    server = ThreadingXMLRPCServer(("0.0.0.0", port), logRequests=False)
    server.register_instance(sl)
    server.register_introspection_functions()
    server.register_multicall_functions()
    server.serve_forever()


if __name__ == '__main__':
    example_sl(8008)


# message_profile.py
#
# Copyright 2019 Eiichi Inohira
# Copyright 2019 Ryota Higashi

import xml.etree.ElementTree as ET
import xml.dom.minidom
import re
import sys

class IO_IdentifiedObject:
    def __init__(self):
        self.identifier = None
        self.name = None

class Parameter_Profile:
    def __init__(self):
        self.name = None
        self.data_type_ref = None
        self.default_value = None
        self.description = None
    def show(self):
        print('Parameter_Profile')
        print('+ name', self.name)
        print('+ data_type_ref', self.data_type_ref)

class Message_Profile:
    def __init__(self):
        self.name = None
        self.type = None
        self.results = []
    def add_result(self, r):
        self.results.append(r)
    def show(self):
        print('Message_Profile')
        print('+ name', self.name)
        print('type', self.type)
        if len(self.results) > 0:
            print('results')
            for x in self.results:
                x.show()

class HRI_Component_Profile(IO_IdentifiedObject):
    def __init__(self):
        self.identifier = None
        self.name = None
        self.parameter = []
        self.subcomponent = []
        self.message = []
    def add_parameter(self, p):
        self.parameter.append(p)
    def add_subcomponent(self, s):
        self.subcomponent.append(s)
    def add_message(self, m):
        self.message.append(m)
    def show(self):
        print('HRI_Component_Profile')
        print('identifier', self.identifier)
        print('name', self.name)
        print('p:s:m',len(self.parameter), len(self.subcomponent), len(self.message))
        for x in self.parameter:
            x.show()
        for x in self.subcomponent:
            x.show()
        for x in self.message:
            x.show()
        print("---")

class HRI_Engine_Profile(IO_IdentifiedObject):
    def __init__(self,identifier=None,name=None):
        self.identifier = identifier
        self.name = name
        self.component = []
        self.sub_profile = []
    def add_component(self, c):
        self.component.append(c)
    def add_engine(self, e):
        self.sub_profile.append(e)
    def show(self):
        for x in self.component:
            x.show()
        for x in self.sub_profile:
            x.show()
    def toxml(self,doc1=None):
        if doc1 is None:
            ET.register_namespace('rois','http://www.omg.org/spec/RoIS/20151201')
            ET.register_namespace('gml','http://www.opengis.net/gml/3.2')
            root = ET.Element('{http://www.omg.org/spec/RoIS/20151201}HRIEngineProfile')
            root.set('{http://www.opengis.net/gml/3.2}id', 'engine_profile')
        else:
            root = ET.SubElement(doc1, 'rois:SubProfile')

        ET.SubElement(root, 'gml:identifier').text = self.identifier
        ET.SubElement(root, 'gml:name').text = self.name
        if len(self.component) >= 1:
            for x in self.component:
                ET.SubElement(root, 'rois:HRIComponent').text = x
                #print(a)
        else:
            pass
        # for x in self.component:
        #         x.toxml(root)
        for x in self.sub_profile:
            x.toxml(root)
        if doc1 is None:
            # ET.dump(root)
            s = ET.tostring(root,encoding='utf-8').decode('utf-8')
            x = xml.dom.minidom.parseString(s)
            return x.toprettyxml()

# class Search_Component():
#     def __init__(self):
#         self.eng_pro = None
#     def analysis(self):
#         file_name = 'engine_profile/' + self.eng_pro + '.xml'
#         tree = ET.parse(file_name)
#         root = tree.getroot()
#         comp_list = []
#         if 'HRIEngineProfile' in root.tag:
#             for x in root:
#                 if 'SubProfile' in x.tag:
#                     for y in x:
#                         if 'HRIComponent' in y.tag:
#                             comp_list.append(y.text.lstrip())
#                 elif 'HRIComponent' in x.tag:
#                     comp_list.append(x.text.lstrip())

#         return comp_list


class Analyze_Engine_Profile:
    def __init__(self, file_name, my_engine_urn=None):
        self.file_name = file_name
        self.my_engine_urn = my_engine_urn
        self.comp_list = []
        self.main_eng_urn = ""
        self.main_comp = []
        self.sub_eng_urn = ""
        self.sub_comp = []
        self.eng_comp_urn = {}
        self.address = self.file_name + '.xml'
        with open(self.address,'r') as f:
            self.profile = f.read()

    def analysis(self):
        # try:
        tree = ET.parse(self.address)
        root = tree.getroot()
        if 'HRIEngineProfile' in root.tag:
            for x in root:
                if 'identifier' in x.tag:
                    self.main_eng_urn = x.text.lstrip()
                elif 'HRIComponent' in x.tag:
                    self.main_comp.append(x.text.lstrip())
                elif 'SubProfile' in x.tag:
                    for y in x:
                        if 'identifier' in y.tag:
                            self.sub_eng_urn = y.text.lstrip()
                        elif 'HRIComponent' in y.tag:
                            self.sub_comp.append(y.text.lstrip())
                    if self.sub_eng_urn == self.my_engine_urn:
                        self.eng_comp_urn = {}
                        self.eng_comp_urn[self.sub_eng_urn] = self.sub_comp
                        return (self.profile, self.eng_comp_urn, self.sub_comp)
                    self.eng_comp_urn[self.sub_eng_urn] = self.sub_comp
                    self.sub_comp = []

                self.eng_comp_urn[self.main_eng_urn] = self.main_comp
        
        if self.my_engine_urn in self.eng_comp_urn.keys() or self.my_engine_urn == None:
            pass
        else:
            print("my_engine_urn error: Specified URN does not exist")
            sys.exit()
        # エンジンとコンポーネントの対応辞書からコンポーネントの一覧リストを作成する方法
        for c in self.eng_comp_urn.values():
            self.comp_list.extend(c)
        return (self.profile, self.eng_comp_urn, self.comp_list)


class Analyze_Component_Profile:
    def __init__(self, comp_name):
        self.comp_name = comp_name
        self.profile = []
        # self.profile_list = {}
    
    def get_profile(self):
        comp_pro = {'SystemInformation':'component_profile_SI','PersonDetection':'component_profile_PD',
                    'PersonLocalization':'component_profile_PL','PersonIdentification':'component_profile_PI',
                    'FaceDetection':'component_profile_FD','FaceLocalization':'component_profile_FL',
                    'SoundDetection':'component_profile_SD','SoundLocalization':'component_profile_SL',
                    'SpeechRecognition':'component_profile_SR','GestureRecognition':'component_profile_GR',
                    'SpeechSynthesis':'component_profile_SS','Reaction':'component_profile_R',
                    'Navigation':'component_profile_N','Follow':'component_profile_F','Move':'component_profile_M'}
        try:
            self.address = 'tests/component_profile/' + comp_pro[self.comp_name] + '.xml'
            with open(self.address,'r') as f:
                self.profile = [f.read()]
        except:
            self.profile = None
        
        # for x in comp_pro.values():
        #     self.profile_list[x] = open('component_profile/' + x + '.xml','r').read()

        # return (self.profile, self.profile_list)
        return self.profile



# class Get_Parameter(): # 同じ種類のコンポーネントで接続するエンジンが違うものを区別できていない
#     # エンジンごとにコンポーネントのフォルダを作るべきか，xmlファイルじゃなくて大きなリスト，2次元は配列で管理するか…
#     def __init__(self, comp_ref):
#         self.comp_ref = comp_ref
#         self.file_name = None
#     def get_parameter(self):
#         # m = re.search(r'(\w+)\:(\w+)\:(\w+)\:(\w+)\:(\w+)\:(\w+)',self.comp_ref)
#         # comp_name = m.group(6)        
#         m = self.comp_ref.split(':')
#         comp_name = m[6]
#         comp_pro = {'SystemInformation':'component_profile_SI','PersonDetection':'component_profile_PD',
#                     'PersonLocalization':'component_profile_PL','PersonIdentification':'component_profile_PI',
#                     'FaceDetection':'component_profile_FD','FaceLocalization':'component_profile_FL',
#                     'SoundDetection':'component_profile_SD','SoundLocalization':'component_profile_SL',
#                     'SpeechRecognition':'component_profile_SR','GestureRecognition':'component_profile_GR',
#                     'SpeechSynthesis':'component_profile_SS','Reaction':'component_profile_R',
#                     'Navigation':'component_profile_N','Follow':'component_profile_F','Move':'component_profile_M'}
#         self.file_name = 'component_profile/' + comp_pro[comp_name] + '.xml'
#         tree = ET.parse(self.file_name)
#         root = tree.getroot()
#         para_list = []
#         if 'HRIComponentProfile' in root.tag:
#             # p = HRI_Component_Profile()
#             for x in root:
#                 # if 'identifier' in x.tag:
#                 #     p.identifier = x.text.lstrip()
#                 # if 'name' in x.tag:
#                 #     p.name = x.text.lstrip()
#                 # if 'MessageProfile' in x.tag:
#                 #     m = Message_Profile()
#                 #     m.name = x.attrib["{http://www.omg.org/spec/RoIS/20151201}name"]
#                 #     m.type = x.attrib["{http://www.w3.org/2001/XMLSchema-instance}type"]
#                 #     p.add_message(m)
#                 #     for y in x:
#                 #         print('y', y.tag, y.attrib)
#                 #         if 'Results' in y.tag:
#                 #             r = Parameter_Profile()
#                 #             r.name = y.attrib["{http://www.omg.org/spec/RoIS/20151201}name"]
#                 #             m.add_result(r)
#                 #             for z in y:
#                 #                 print(z.tag, z.attrib)
#                 #                 if 'data_type_ref' in z.tag:
#                 #                     r.data_type_ref = z.attrib["{http://www.omg.org/spec/RoIS/20151201}code"]
#                 if 'ParameterProfile' in x.tag:
#                     pp = Parameter_Profile()
#                     pp.name = x.attrib["{http://www.omg.org/spec/RoIS/20151201}name"]
#                     pp.default_value = x.attrib["{http://www.omg.org/spec/RoIS/20151201}default_value"]
#                     pp.description = x.attrib["{http://www.omg.org/spec/RoIS/20151201}description"]
#                     for y in x:
#                         if 'data_type_ref' in y.tag:
#                             pp.data_type_ref = y.attrib["{http://www.omg.org/spec/RoIS/20151201}code"]
#                     # exec("para%d = [%s,%s,%s]" %(x,pp.name,pp.data_type_ref,pp.default_value))
#                     # para_list.append("para%d"%(x))
#                     para_list.append(pp.name)
#                     para_list.append(pp.data_type_ref)
#                     para_list.append(pp.default_value)

#         return para_list

# class Set_Parameter():
#     def __init__(self, comp_ref, para):
#         self.comp_ref = comp_ref
#         self.para = para
#     def set_parameter(self):
#         a = Get_Parameter(self.comp_ref)
#         a.get_parameter()
#         tree = ET.parse(a.file_name)
#         root = tree.getroot()
#         if 'HRIComponentProfile' in root.tag:
#             for x in root:
#                 if 'ParameterProfile' in x.tag:
#                     x.set("{http://www.omg.org/spec/RoIS/20151201}name",self.para[x][0])
#                     x.set("{http://www.omg.org/spec/RoIS/20151201}default_value",self.para[x][2])
#                     # x.attrib["{http://www.omg.org/spec/RoIS/20151201}description"]
#                     for y in x:
#                         if 'data_type_ref' in y.tag:
#                             y.set("{http://www.omg.org/spec/RoIS/20151201}code",self.para[x][1])
        



if __name__ == "__main__":
    # test = Get_Parameter("urn:x-rois:def:HRIComponent:Kyutech:eng1:SpeechRecognition") # ":(none):"の時対応できない 文字の指定を(\s+)にすればスペースは検索できる
    # print(test.get_parameter())
    # print(test.file_name)
    # test = Set_Parameter("urn:x-rois:def:HRIComponent:Kyutech:eng1:SpeechRecognition",
    # [['rule', 'string', 'default'], ['languages', 'string[]', 'jp'], ['grammar', 'string', 'default']])
    # test.set_parameter()

    test = Analyze_Engine_Profile("engine_profile_2","urn:x-rois:def:HRIEngine:Kyutech::SubEng01")
    (profile,eng_comp,comp_list) = test.analysis()
    print(profile, eng_comp, comp_list)

    test2 = Analyze_Engine_Profile("engine_profile_2","")
    (profile,eng_comp,comp_list) = test2.analysis()
    print(profile, eng_comp, comp_list)


import message_profile_test
import xml.etree.ElementTree as ET

tree = ET.parse('engine_profile/engine_profile_binding.xml')
root = tree.getroot()
ET.register_namespace('rois','http://www.omg.org/spec/RoIS/20151201')
ET.register_namespace('gml','http://www.opengis.net/gml/3.2')

doc1 = ET.Element('{http://www.omg.org/spec/RoIS/20151201}HRIEngineProfile')
# doc2 = ET.Element('{http://www.omg.org/spec/RoIS/20151201}SubProfile')
# doc3 = ET.Element('{http://www.opengis.net/gml/3.2}name')
doc1.set('{http://www.opengis.net/gml/3.2}id', 'engine_profile')

# sub_pro = ET.SubElement(doc1, '{http://www.omg.org/spec/RoIS/20151201}SubProfile')
# sub_id = ET.SubElement(sub_pro, '{http://www.opengis.net/gml/3.2}identifier')
# sub_id.text = ('urn:x-rois:def:HRIEngine:ATR::SubEng01')
# ET.dump(doc1)

e = message_profile_test.HRI_Engine_Profile()

# print(root.tag, root.attrib)
for x in root:
    if 'identifier' in x.tag:
        # x.text = e.identifier
        x.text = "urn:x-rois:def:HRIEngine:ATR::MainHRI"
    if 'name' in x.tag:
    #     x.text = e.name
        x.text = "mainHRI"
    if len(e.sub_profile) >= 1:
        s = message_profile_test.HRI_Engine_Profile()
        sub_pro = ET.SubElement(doc1, '{http://www.omg.org/spec/RoIS/20151201}SubProfile')
        print('OK')

        # for a in e.sub_profile:
        #     sub_pro = ET.SubElement(doc1, '{http://www.omg.org/spec/RoIS/20151201}SubProfile')
         
           
    #         sub_id = ET.SubElement(doc1,'gml:identifier')
    #         sub_name = ET.SubElement(doc1,'gml:name')
    #         for y in x:    
    #             if 'identifier' in y.tag:
    #                 y.text = s.identifier[y-1] #要確認
    #             if 'name' in y.tag:
    #                 y.text = s.name #message_profileに関数/配列を追加する必要あり
    #                 for b in s.component:
    #                     comp_id = ET.SubElement('doc2,'rois:HRIComponent')
    #                 if 'HRIComponent' in y.tag:
    #                     y.text = s.component[y-1]
    else:
        pass
    #     for c in e.component:
    #         comp_id2 = ET.SubElement(a,'rois:HRIComponent')
    #     if 'HRIComponent' in x.tag:
    #         x.text = s.component[x-1]

test = ET.ElementTree(doc1)
test.write('engine_profile/engine_profile_binding_2.xml',xml_declaration=True,encoding="UTF-8",short_empty_elements=False)
# tree = ET.ElementTree(root)
# tree.write('engine_profile/engine_profile_binding_2.xml',xml_declaration=True,encoding="UTF-8")


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyRoIS-common",
    version="0.0.5",
    author="Eiichi Inohira",
    author_email="inohira.eiichi402@mail.kyutech.jp",
    description="Implementation of HRI Engine on RoIS Framework in Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://inohira.mns.kyutech.ac.jp/git/inohira/pyRoIS-higashi",
    packages=setuptools.find_packages(),
    install_requires="pyrois",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)

# pip3 install --upgrade pip
# pip3 install twine
# python3 setup.py sdist
# python3 -m twine upload --repository-url https://test.pypi.org/legacy/ dist/*
# python3 -m twine upload --repository-url https://pypi.org/ dist/*



# test_execute.py
#
# Copyright 2020 Ryota Higashi
# Copyright 2021 Eiichi Inohira
#
# This software may be modified and distributed under the terms
# of the MIT license
#
# for python 3

"""unittest サブエンジンが存在しない場合のテスト
"""

import time
from multiprocessing import Process, Queue
import unittest
import xmlrpc.client
import sys
import re
import logging
import platform
from datetime import datetime
import atexit

from pyrois import RoIS_HRI, RoIS_Common, RoIS_Service
from pyrois_common import HRI_Engine_sample, HRI_Engine_client_sample, command_message
from . import utils

from pyrois_common import System_Information
from pyrois_common import Person_Detection
from pyrois_common import Person_Localization
from pyrois_common import Person_Identification
from pyrois_common import Face_Detection
from pyrois_common import Face_Localization
from pyrois_common import Sound_Detection
from pyrois_common import Sound_Localization
from pyrois_common import Speech_Recognition
from pyrois_common import Gesture_Recognition
from pyrois_common import Speech_Synthesis
from pyrois_common import Reaction
from pyrois_common import Navigation
from pyrois_common import Follow
from pyrois_common import Move


eng_profile = """<?xml version="1.0" encoding="UTF-8"?>
<!-->
Robotic Interaction Service Framework Version 1.2
    > Annex A: Examples of Profile in XML
        > A.4 HRI Engine Profile (p.102 (PDF:p.115)) reference
<-->

<rois:HRIEngineProfile gml:id="engine_profile"
    xmlns:rois="http://www.omg.org/spec/RoIS/20151201"
    xmlns:gml="http://www.opengis.net/gml/3.2">
    <gml:identifier>urn:x-rois:def:HRIEngine:Kyutech::main</gml:identifier>
    <gml:name>main</gml:name>
    <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:main:PersonDetection</rois:HRIComponent>
    <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization</rois:HRIComponent>
    <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:main:SystemInformation</rois:HRIComponent>
</rois:HRIEngineProfile>"""

commnd_unit_list ="""{
    "command_unit_list": [
        {
            "component_ref": "urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization",
            "command_type": "set_parameter",
            "command_id": "main-exe-1-1",
            "arguments": {
                "parameters": [
                    {
                        "name": "detection_threshold",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": 555
                    },
                    {
                        "name": "minimum_interval",
                        "data_type_ref": {
                            "codebook_reference": "",
                            "version": ""
                        },
                        "value": 222
                    }
                ]
            },
            "delay_time": 5
        }
    ]
}"""

right_event_result = {
    (('main-2', RoIS_Service.Completed_Status.OK), 'completed'), # completed
    (('main-exe-1-1', RoIS_Service.Completed_Status.OK), 'completed') # completed
    # (('main-1', 'person_localized', '1'), 'notify_event'),
    # (('main-2', 'person_localized', '1'), 'notify_event'),
    # (('main-3', 'person_detected', '2'), 'notify_event'),
    # (('main-4', 'person_detected', '2'), 'notify_event')
}

right_command_result = [
    RoIS_HRI.ReturnCode_t.OK,
    # (RoIS_HRI.ReturnCode_t.OK, eng_profile),
    # (RoIS_HRI.ReturnCode_t.OK,["urn:x-rois:def:HRIComponent:Kyutech:main:PersonDetection",
    # "urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization",
    # "urn:x-rois:def:HRIComponent:Kyutech:main:SystemInformation"
    # ]),
    # (RoIS_HRI.ReturnCode_t.OK,["urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization",]),
    RoIS_HRI.ReturnCode_t.OK, # bind:PersonLocalization
    # (RoIS_HRI.ReturnCode_t.OK,[1000
    # ,20]), # get_parameter:PersonLocalization
    (RoIS_HRI.ReturnCode_t.OK,"main-2"), # set_parameter:PersonLocalization
    (RoIS_HRI.ReturnCode_t.OK,commnd_unit_list), # execute
    # (RoIS_HRI.ReturnCode_t.OK,"1"), # subscribe:PersonLocalizationn
    (RoIS_HRI.ReturnCode_t.OK,["Set_parameter(urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization)"]), # get_command_result
    RoIS_HRI.ReturnCode_t.OK # disconnect
    ]
# dt = datetime.now()
# dt_format = dt.strftime("%Y%m%d-%H%M")
# logger1 = logging.getLogger('[main]_Engine: HRI_Engine_sample.py')
# logger1.setLevel(logging.DEBUG)
# logger2 = logging.getLogger('Service_Application: HRI_Engine_client_sample.py')
# logger2.setLevel(logging.DEBUG)

# formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
# f = logging.FileHandler('unittest_execute_{}.log'.format(dt_format))
# f.setLevel(logging.DEBUG)
# f.setFormatter(formatter)
# logger1.addHandler(f)
# logger2.addHandler(f)

# c = logging.StreamHandler()
# c.setLevel(logging.ERROR)
# c.setFormatter(formatter)
# logger1.addHandler(c)
# logger2.addHandler(c)

def test_sa(q):
    c_ri = command_message.RoIS_Identifier("","")
    c_p1 = command_message.Parameter("detection_threshold", c_ri, 555)
    c_p2 = command_message.Parameter("minimum_interval", c_ri, 222)
    c_al = command_message.ArgumentList([c_p1,c_p2])
    # c_cb = command_message.CommandBase(5) # 反映されない気がする ソースいじる必要がある？
    c_cm = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization',"set_parameter","",c_al) # command_id引数いらない
    c_cm.delay_time = 5
    c_cus = command_message.CommandUnitSequence([c_cm])

    i = HRI_Engine_client_sample.IF("http://127.0.0.1:8016")
    res = [
        i.connect(),
        # i.get_profile(""),
        # i.search(""),
        # i.bind('urn:x-rois:def:HRIComponent:Kyutech:main:PersonDetection'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization'),
        # i.bind('urn:x-rois:def:HRIComponent:Kyutech:main:SystemInformation'),
        # i.get_parameter('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization'),
        i.set_parameter('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization',[100,10]),
        i.execute(c_cus),
        # i.subscribe("person_localized",""),
        # i.subscribe("person_detected",""),
        i.get_command_result("main-2","")
    ]
        
    # print(res)

    s_e = set()
    for x in range(2):
        e = i.get_event()
        s_e.add((e[0][:3],e[1]))
    
    res.extend([
        i.disconnect()
    ])

    r = 0
    if s_e != right_event_result:
        print(s_e)
        r=1
    if res != right_command_result:
        print(res)
        r=2
    q.put(r)
    # print('sa finished')
    return r

def kill_ps(ps):
    # terminate the server process
    for x in ps:
        x.kill()
        time.sleep(0.5)

class Execute_test(unittest.TestCase):    
    maxDiff = None
    
    def setUp(self):
        """ setUP
        """
        # start the server process
        self.q = Queue()
        self.ps = [Process(target=test_sa, args=(self.q,)),
            Process(target=HRI_Engine_sample.test_engine, args=(8016,"tests/engine_profile/engine_profile_3_1", "client_info1", "urn:x-rois:def:HRIEngine:Kyutech::main", None,)), # loggerを代入するとエラー self.fhを持っていないから？
                        # File "C:\Users\higashi\Documents\pyRoIS_higashi\HRI_Engine_sample.py", line 780, in init_logger
                        # self.logger_client.addHandler(self.fh)
                        # AttributeError: 'IF' object has no attribute 'fh'
            # Process(target=Person_Detection.example_pd, args=(8002,)),
            Process(target=Person_Localization.example_pl, args=(8003,))
            # Process(target=System_Information.example_si, args=(8001,))
            ]
        
        for x in reversed(self.ps[1:]):
            x.start()
            time.sleep(0.5)

        time.sleep(5.0)
        

    def test_execute(self):
        """ test_execute
        """
        self.ps[0].start()
        r = self.q.get(60)
        return self.assertEqual(r,0)

    def tearDown(self):
        """ tearDown
        """
        atexit.register(kill_ps, self.ps)


if __name__ == '__main__':
    # ip = sys.argv
    # if len(ip) > 1:
    #     m = re.search(r'(\d+)\.(\d+)\.(\d+)\.(\d+)',ip[1])
    # else:
    #     m = None
    # if m and len(m.groups()) == 4:
    #     ser_uri = "http://" + ip[1] +":8000"
    #     print("server_url == " + ser_uri + "\n")
    #     del sys.argv[1]
    # else:
    #     ser_uri = "http://127.0.0.1:8000"
    #     print("server_url == " + ser_uri + "\n")
    #     if len(ip) > 1 and ip[1][0] != '-':
    #         del sys.argv[1]

    unittest.main()


# HRI_Engine_testNEW.py
#
# Copyright 2017 Eiichi Inohira
# Addition 2018 Ryota Higashi 
# This software may be modified and distributed under the terms
# of the MIT license
#
# For python 3
# For HRI Engine

"""HRI_Engine_example
"""

from __future__ import print_function

import os
import sys
import xml.etree.ElementTree as ET
import glob
import unittest

from pyrois import RoIS_HRI


class TestProfile(unittest.TestCase):
    def test_search(self):
        a = self.search('component_profile_FL')
        return self.assertNotEqual(a[1],["None"])
    def search(self, condition):
        status = RoIS_HRI.ReturnCode_t.OK.value #xmlで接続したい記述．それを探してIDを返す
        component_ref_list = ["None"]
            # List< RoIS_Identifier>
            # とりあえず，conditionに記載したコンポーネント名をxmlファイルの中から探す
        file_list = glob.glob('tests/component_profile/*.xml')
        # print(os.getcwd())
        filename = os.path.join('tests','component_profile',condition + '.xml')
        # print(filename)
        if filename in file_list:
            with open(filename,'r') as f:
                self.component_profile = f.read()
            tree = ET.parse(filename)
            root = tree.getroot()
            text = root.attrib
            component_ref_list = list(text.values())
            # print(component_ref_list)
            
        return (status, component_ref_list)

if __name__ == "__main__":
    unittest.main()



import message_profile
import xml.etree.ElementTree as ET
import xml.dom.minidom


ET.register_namespace('rois','http://www.omg.org/spec/RoIS/20151201')
ET.register_namespace('gml','http://www.opengis.net/gml/3.2')


e = message_profile.HRI_Engine_Profile(
identifier = "urn:x-rois:def:HRIEngine:Kyutech::mainEng",
name = "mainEng"
)

# comp0 = ["urn:x-rois:def:HRIComponent:Kyutech:main:SystemInformation",
#         "urn:x-rois:def:HRIComponent:Kyutech:main:Move"]
# for x in comp0:
#     e.add_component(x)

s1 = message_profile.HRI_Engine_Profile(
identifier = "urn:x-rois:def:HRIEngine:Kyutech::SubEng01",
name = "subEng01"
)
e.add_engine(s1)

# s1.add_component("urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonDetection")
# s1.add_component("urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonIdentification")
c1 = message_profile.HRI_Component_Profile()
comp1 = ["urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonDetection",
        "urn:x-rois:def:HRIComponent:Kyutech:sub01:PersonIdentification"]
for x in comp1:
    s1.add_component(x)
    c1.identifier = x
    c1.show()



s2 = message_profile.HRI_Engine_Profile(
identifier = "urn:x-rois:def:HRIEngine:Kyutech::SubEng02",
name = "subEng02"
)
e.add_engine(s2)

c2 = message_profile.HRI_Component_Profile()
comp2 = ["urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceDetection",
        "urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization",
        "urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation"]
for x in comp2:
    s2.add_component(x)
    c2.identifier = x
    c2.show()

print(e.toxml())




# command_message_test.py

import command_message
import json
import itertools

# c_ri = command_message.RoIS_Identifier("","")
# c_p1 = command_message.Parameter("detection_threshold", c_ri, 555)
# c_p2 = command_message.Parameter("minimum_interval", c_ri, 222)
# c_al = command_message.ArgumentList([c_p1,c_p2])
# c_cb = command_message.CommandBase(5) # 反映されない気がする ソースいじる必要がある？
# c_cm = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization',"set_parameter","",c_al) # command_id引数いらない
# c_cm.delay_time = 5
# c_b = command_message.Branch([c_cm])
# c_cc = command_message.ConcurrentCommands([c_b])
# c_cc.delay_time =10
# c_cus = command_message.CommandUnitSequence([c_cm,c_cc])
# print(c_cus.__dict__)
# print(c_cus.command_unit_list[0].__class__.__name__)
# print(vars(c_cus.command_unit_list[0].arguments))
# # print(vars(c_cus.command_unit_list[0].arguments[0]))

def object_to_json(item):
    if isinstance(item, object) and hasattr(item, '__dict__'):
        return item.__dict__
    else:
        raise TypeError



# j=json.dumps(c_cus, default=object_to_json, indent=4)
# print(j)

# obj=json.loads(j)
# print(obj)

# class ObjectLike(dict):
#     __getattr__ = dict.get
  
# obj = json.loads(j, object_hook=ObjectLike)
# print(obj)
# print(obj.command_unit_list[0].arguments.parameters[0].name)


c_ri = command_message.RoIS_Identifier("","")
c_p_ss1 = command_message.Parameter("speech_text", c_ri, "aaa")
c_p_ss2 = command_message.Parameter("SSML_text", c_ri, "ssml")
c_p_ss3 = command_message.Parameter("volume", c_ri, 50)
c_p_ss4 = command_message.Parameter("languages", c_ri, "japanese")
c_p_ss5 = command_message.Parameter("character", c_ri, "m1")
c_al1 = command_message.ArgumentList([c_p_ss1,c_p_ss2,c_p_ss3,c_p_ss4,c_p_ss5])
c_cm1 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis',"set_parameter","",c_al1)
c_cm1.delay_time = 0

c_p = command_message.Parameter("", c_ri, "")
c_al2 = command_message.ArgumentList([c_p])
c_cm2 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:sub01:SpeechSynthesis',"get_parameter","",c_al2)
c_cm2.delay_time = 0

c_p_fl1 = command_message.Parameter("detection_threshold", c_ri, 3)
c_p_fl2 = command_message.Parameter("minimum_interval", c_ri, 2)
c_al3_1_1 = command_message.ArgumentList([c_p_fl1,c_p_fl2])
c_cm3_1_1 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization',"set_parameter","",c_al3_1_1)
c_cm3_1_1.delay_time = 3000

c_p_n1 = command_message.Parameter("target_position", c_ri, [100.0,200.0])
c_p_n2 = command_message.Parameter("time_limit", c_ri, 60000)
c_p_n3 = command_message.Parameter("routing_policy", c_ri, "time priority")
c_al3_1_2 = command_message.ArgumentList([c_p_n1,c_p_n2,c_p_n3])
c_cm3_1_2 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:sub02:Navigation',"set_parameter","",c_al3_1_2)
c_cm3_1_2.delay_time = 4000

c_p_m1 = command_message.Parameter("line", c_ri, [1000,360])
c_p_m2 = command_message.Parameter("curve", c_ri, [1000,90])
c_p_m3 = command_message.Parameter("time", c_ri, 1000)
c_al3_2 = command_message.ArgumentList([c_p_m1,c_p_m2,c_p_m3])
c_cm3_2 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:sub03:Move',"set_parameter","",c_al3_2)
c_cm3_2.delay_time = 5000
# c_p_fl3 = command_message.Parameter("detection_threshold", c_ri, 3)
# c_p_fl4 = command_message.Parameter("minimum_interval", c_ri, 2)
# c_al3_2 = command_message.ArgumentList([c_p_fl3,c_p_fl4])
# c_cm3_2 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:sub02:FaceLocalization',"set_parameter","",c_al3_2)
# c_cm3_2.delay_time = 3000

c_b1 = command_message.Branch([c_cm3_1_1,c_cm3_1_2])
c_b2 = command_message.Branch([c_cm3_2])
c_cc = command_message.ConcurrentCommands([c_b1,c_b1,c_b2,c_b2])
c_cc.delay_time = 0

c_p_f1 = command_message.Parameter("target_object_ref", c_ri, "human1")
c_p_f2 = command_message.Parameter("distance", c_ri, 500)
c_p_f3 = command_message.Parameter("time_limit", c_ri, 60000)
c_al4 = command_message.ArgumentList([c_p_f1,c_p_f2,c_p_f3])
c_cm4 = command_message.CommandMessage('urn:x-rois:def:HRIComponent:Kyutech:sub01:Follow',"set_parameter","",c_al4)
c_cm4.delay_time = 1000    

c_cus1 = command_message.CommandUnitSequence([c_cm1,c_cm2,c_cc,c_cc,c_cm4])

j=json.dumps(c_cus1, default=object_to_json, indent=4)
d=json.loads(j)

# print(d)
# print(next(iter(d.values())))

# d = {}

# 変数名は要変更
# if next(iter(d)) == "command_unit_list":
branch_list_num = 0
is_correct = True
if "command_unit_list" in d.keys():
    command_list = d["command_unit_list"]
    for command in command_list:
        if "branch_list" in command.keys():
            branch_list_num += 1
            branch_num = 0
            all_branch_comprefs = {}
            for branch in command["branch_list"]:
                branch_num += 1
                branch_command_list = branch["command_list"]
                component_refs = [branch_command["component_ref"] for branch_command in branch_command_list if branch_command["command_type"] == "set_parameter"]
                all_branch_comprefs[branch_num] = component_refs
            print(all_branch_comprefs)
            for combi in itertools.combinations(all_branch_comprefs.keys(),2):
                # print(combi)
                common = set(all_branch_comprefs[combi[0]]).intersection(set(all_branch_comprefs[combi[1]]))
                # print(common)
                if len(common) > 0:
                    is_correct = False
                    print("Execute(~~~) [BranchList_num:{}, Branch_Combination:{}, Same_Components:{}]: Cannot access the set_parameter in the same component from different branches.".format(branch_list_num,combi,common))
    if is_correct == False:
        print("False")
    else:
        print("True")


# Service_Application_Base_example.py
#
# Copyright 2017 Eiichi Inohira
# Copyright 2019 Ryota Higashi
# This software may be modified and distributed under the terms
# of the MIT license
#
# for python 3
# for HRI Engine

"""Service_Application_Base_example
"""

import sys
import queue
import time
import threading
import logging

from pyrois import RoIS_Service

import socketserver
import xmlrpc.server

class ThreadingXMLRPCServer(socketserver.ThreadingMixIn, xmlrpc.server.SimpleXMLRPCServer):
    """ThreadingXMLRPCServer
    """
    pass


class Service_Application_Base(RoIS_Service.Service_Application_Base):
    """Service_Application_Base
    """
    def __init__(self, logger=None):
        self.event_queue = queue.Queue()
        if logger is not None:
            self.logger = logger

    def poll_event(self):
        """poll_event
        """
        # print("invoked poll")
        self.logger.info("invoked poll")
        msg = self.event_queue.get()
        return msg

    def completed(self, command_id, status):
        # print('completed')
        self.logger.info('***completed***')
        msg = xmlrpc.client.dumps((command_id, status), 'completed')
        self.event_queue.put(msg)

    def notify_error(self, error_id, error_type):
        # print('notify_error')
        self.logger.info('***notify_error***')
        msg = xmlrpc.client.dumps((error_id, error_type), 'notify_error')
        self.event_queue.put(msg)

    def notify_event(self, event_id, event_type, subscribe_id, expire):
        # print('notify_event')
        self.logger.info('***notify_event***')
        msg = xmlrpc.client.dumps(
            (event_id, event_type, subscribe_id, expire), 'notify_event')
        self.event_queue.put(msg)




# def example_sa(port):
#     """example_sa
#     """
#     logger = logging.getLogger('Service_Application_Base_for_sub_HRI_engine')
#     logger.setLevel(logging.DEBUG)
#     # ch = logging.StreamHandler()
#     ch = logging.FileHandler('event_notification_test.log') 
#     ch.setLevel(logging.DEBUG)
#     formatter = logging.Formatter(
#         '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
#     ch.setFormatter(formatter)
#     logger.addHandler(ch)

#     sa = Service_Application_Base(logger=logger)

#     # start the timer to dispatch events
#     # t = threading.Timer(5, event_dispatch, args=(sa,))
#     # t.start()

#     # start the XML-RPC server
#     server = ThreadingXMLRPCServer(("0.0.0.0", port), logRequests=False)
#     server.register_instance(sa)
#     server.register_introspection_functions()
#     server.register_multicall_functions()
#     # print("server running")
#     server.serve_forever()


# if __name__ == '__main__':
#     example_sa(8016)



# test_main_sub_engine3.py
#
# Copyright 2020 Ryota Higashi
# Copyright 2021 Eiichi Inohira
#
# This software may be modified and distributed under the terms
# of the MIT license
#
# for python 3

"""unittest サブエンジンが存在しない場合のテスト
"""

import time
from multiprocessing import Process, Queue
import unittest
import xmlrpc.client
import sys
import re
import logging
import platform
import atexit

from pyrois import RoIS_HRI, RoIS_Common, RoIS_Service
from pyrois_common import HRI_Engine_sample, HRI_Engine_client_sample


from pyrois_common import System_Information
from pyrois_common import Person_Detection
from pyrois_common import Person_Localization
from pyrois_common import Person_Identification
from pyrois_common import Face_Detection
from pyrois_common import Face_Localization
from pyrois_common import Sound_Detection
from pyrois_common import Sound_Localization
from pyrois_common import Speech_Recognition
from pyrois_common import Gesture_Recognition
from pyrois_common import Speech_Synthesis
from pyrois_common import Reaction
from pyrois_common import Navigation
from pyrois_common import Follow
from pyrois_common import Move


eng_profile = """<?xml version="1.0" encoding="UTF-8"?>
<!-->
Robotic Interaction Service Framework Version 1.2
    > Annex A: Examples of Profile in XML
        > A.4 HRI Engine Profile (p.102 (PDF:p.115)) reference
<-->

<rois:HRIEngineProfile gml:id="engine_profile"
    xmlns:rois="http://www.omg.org/spec/RoIS/20151201"
    xmlns:gml="http://www.opengis.net/gml/3.2">
    <gml:identifier>urn:x-rois:def:HRIEngine:Kyutech::main</gml:identifier>
    <gml:name>main</gml:name>
    <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:main:PersonDetection</rois:HRIComponent>
    <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization</rois:HRIComponent>
    <rois:HRIComponent>urn:x-rois:def:HRIComponent:Kyutech:main:SystemInformation</rois:HRIComponent>
</rois:HRIEngineProfile>"""
     
comp_profile = """<?xml version="1.0" encoding="UTF-8"?>
<!-->
Robotic Interaction Service Framework Version 1.2
    > 9.3 XML PSM : System Information (p.85 (PDF:p.98))
<-->

<rois:HRIComponentProfile gml:id="component_profile"
    xmlns:rois="http://www.omg.org/spec/RoIS/20151201"
    xmlns:ns2="http://www.w3.org/1999/xlink"
    xmlns:gml="http://www.opengis.net/gml/3.2"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
    <gml:identifier codeSpace="urn:ietf:rfc:2141">
        urn:x-rois:def:component:OMG::SystemInformation</gml:identifier>
    <gml:name>system_info</gml:name>
    <!-- ===== Query Messages ===== -->
    <rois:MessageProfile rois:name="robot_position" xsi:type="rois:QueryMessageProfileType">
        <rois:Results rois:description="position of robot or its parts in comma seperateddouble values [x, y, th]" 
            rois:name="position_data">
            <rois:data_type_ref rois:code="String[]"/>
        </rois:Results>
        <rois:Results rois:description="List of robot IDs" rois:name="robot_ref">
            <rois:data_type_ref rois:code="RoISIdentifier[]"/>
        </rois:Results>
        <rois:Results rois:description="timestamp of measurement" rois:name="timestamp">
            <rois:data_type_ref rois:code="DateTime"/>
        </rois:Results>
    </rois:MessageProfile>
    <rois:MessageProfile rois:name="engine_status"
        xsi:type="rois:QueryMessageProfileType">
        <rois:Results rois:description="Operable time of the HRI Engine that includes this basic component" 
            rois:name="operable_time">
            <rois:data_type_ref rois:code="DateTime"/>
        </rois:Results>
        <rois:Results rois:description="Status information of this engine" rois:name="status">
            <rois:data_type_ref rois:code="Component_Status"/>
        </rois:Results>
    </rois:MessageProfile>
</rois:HRIComponentProfile>"""

right_event_result = {
    (('main-6', RoIS_Service.Completed_Status.OK), 'completed'), # completed
    (('main-1', 'person_localized', '1'), 'notify_event'),
    (('main-2', 'person_localized', '1'), 'notify_event'),
    (('main-3', 'person_detected', '2'), 'notify_event'),
    (('main-4', 'person_detected', '2'), 'notify_event')
}

right_command_result = [
    RoIS_HRI.ReturnCode_t.OK,
    
    (RoIS_HRI.ReturnCode_t.OK, eng_profile),
    
    (RoIS_HRI.ReturnCode_t.OK, [comp_profile]),

    (RoIS_HRI.ReturnCode_t.OK,["urn:x-rois:def:HRIComponent:Kyutech:main:PersonDetection",
    "urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization",
    "urn:x-rois:def:HRIComponent:Kyutech:main:SystemInformation"
    ]),

    RoIS_HRI.ReturnCode_t.OK, # bind:PersonDetection
    RoIS_HRI.ReturnCode_t.OK, # bind:PersonLocalization
    RoIS_HRI.ReturnCode_t.OK, # bind:SystemInformation

    (RoIS_HRI.ReturnCode_t.OK,[1000,20]), # get_parameter:PersonLocalization
    (RoIS_HRI.ReturnCode_t.OK,"main-6"), # set_parameter:PersonLocalization

    (RoIS_HRI.ReturnCode_t.OK,"1"), # subscribe:PersonLocalizationn
    (RoIS_HRI.ReturnCode_t.OK,"2"), # subscribe:PersonDetection
    (RoIS_HRI.ReturnCode_t.OK,['Bind(urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization):Success']), # get_command_result
    
    
    (RoIS_HRI.ReturnCode_t.OK,[100,10]), # get_parameter:PersonLocalization
    RoIS_HRI.ReturnCode_t.OK # disconnect
    ]

def test_sa(q):
    i = HRI_Engine_client_sample.IF("http://127.0.0.1:8016")
    res = [
        i.connect(),
        i.get_profile(""),
        i.query("SystemInformation",""),
        i.search(""),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:main:PersonDetection'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization'),
        i.bind('urn:x-rois:def:HRIComponent:Kyutech:main:SystemInformation'),
        i.get_parameter('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization'),
        i.set_parameter('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization',[100,10]),
        i.subscribe("person_localized",""),
        i.subscribe("person_detected",""),
        i.get_command_result("main-3","")
    ]
        
    s_e = set()
    for x in range(5):
        e = i.get_event()
        s_e.add((e[0][:3],e[1]))
    
    res.extend([i.get_parameter('urn:x-rois:def:HRIComponent:Kyutech:main:PersonLocalization'),
        i.disconnect()
    ])

    r = 0
    if s_e != right_event_result:
        print('event results were wrong')
        print(s_e - right_event_result)
        r=1
    if res != right_command_result:
        for x, y in zip(res,right_command_result):
            if x != y:
                print('*', x, y)
        r+=10
    q.put(r)
    # print('sa finished')
    return r
    
def kill_ps(ps):
    # terminate the server process
    for x in ps:
        x.kill()
        time.sleep(0.5)

class Main_Sub_Engine_test(unittest.TestCase):    
    maxDiff = None
    
    def setUp(self):
        """ setUP
        """

        # start the server process
        self.q = Queue()
        self.ps = [Process(target=test_sa, args=(self.q,)),
            Process(target=HRI_Engine_sample.test_engine, args=(8016,"tests/engine_profile/engine_profile_3", "client_info1", None, None,)),
            Process(target=Person_Detection.example_pd, args=(8002,)),
            Process(target=Person_Localization.example_pl, args=(8003,)),
            Process(target=System_Information.example_si, args=(8001,))
            ]
        
        for x in reversed(self.ps[1:]):
            x.start()
            time.sleep(0.5)

        time.sleep(5.0)
        

    def test_IF(self):
        """ test_IF
        """
        self.ps[0].start()
        r = self.q.get(60)
        return self.assertEqual(r,0)

    def tearDown(self):
        """ tearDown
        """
        atexit.register(kill_ps, self.ps)

if __name__ == '__main__':
    # ip = sys.argv
    # if len(ip) > 1:
    #     m = re.search(r'(\d+)\.(\d+)\.(\d+)\.(\d+)',ip[1])
    # else:
    #     m = None
    # if m and len(m.groups()) == 4:
    #     ser_uri = "http://" + ip[1] +":8000"
    #     print("server_url == " + ser_uri + "\n")
    #     del sys.argv[1]
    # else:
    #     ser_uri = "http://127.0.0.1:8000"
    #     print("server_url == " + ser_uri + "\n")
    #     if len(ip) > 1 and ip[1][0] != '-':
    #         del sys.argv[1]

    unittest.main()